@@ -76,8 +76,8 @@ build_time_vars = {'ABIFLAGS': '',
  'CONFINCLUDEDIR': '/usr/include',
  'CONFINCLUDEPY': '/usr/include/python3.10',
  'COREPYTHONPATH': '',
- 'COVERAGE_INFO': '/build/python3.10-mmQOQh/python3.10-3.10.6/build-shared/coverage.info',
- 'COVERAGE_REPORT': '/build/python3.10-mmQOQh/python3.10-3.10.6/build-shared/lcov-report',
+ 'COVERAGE_INFO': '/build/python3.10-cD1wDb/python3.10-3.10.7/build-shared/coverage.info',
+ 'COVERAGE_REPORT': '/build/python3.10-cD1wDb/python3.10-3.10.7/build-shared/lcov-report',
  'COVERAGE_REPORT_OPTIONS': '--no-branch-coverage --title "CPython lcov '
                             'report"',
  'CPPFLAGS': '-IObjects -IInclude -IPython -I. -I../Include -Wdate-time '
@@ -779,7 +779,7 @@ build_time_vars = {'ABIFLAGS': '',
  'READELF': 'readelf',
  'RESSRCDIR': 'Mac/Resources/framework',
  'RETSIGTYPE': 'void',
- 'RUNSHARED': 'LD_LIBRARY_PATH=/build/python3.10-mmQOQh/python3.10-3.10.6/build-shared',
+ 'RUNSHARED': 'LD_LIBRARY_PATH=/build/python3.10-cD1wDb/python3.10-3.10.7/build-shared',
  'SCRIPTDIR': '/usr/lib',
  'SETPGRP_HAVE_ARG': 0,
  'SHELL': '/bin/sh',
@@ -821,10 +821,10 @@ build_time_vars = {'ABIFLAGS': '',
  'TCLTK_LIBS': '-ltk8.6 -ltkstub8.6 -ltcl8.6 -ltclstub8.6',
  'TESTOPTS': '',
  'TESTPATH': '',
- 'TESTPYTHON': 'LD_LIBRARY_PATH=/build/python3.10-mmQOQh/python3.10-3.10.6/build-shared '
+ 'TESTPYTHON': 'LD_LIBRARY_PATH=/build/python3.10-cD1wDb/python3.10-3.10.7/build-shared '
                './python',
  'TESTPYTHONOPTS': '',
- 'TESTRUNNER': 'LD_LIBRARY_PATH=/build/python3.10-mmQOQh/python3.10-3.10.6/build-shared '
+ 'TESTRUNNER': 'LD_LIBRARY_PATH=/build/python3.10-cD1wDb/python3.10-3.10.7/build-shared '
                './python ../Tools/scripts/run_tests.py',
  'TESTSUBDIRS': 'ctypes/test \\',
  'TESTTIMEOUT': 1200,
@@ -853,8 +853,8 @@ build_time_vars = {'ABIFLAGS': '',
  'WITH_VALGRIND': 0,
  'X87_DOUBLE_ROUNDING': 0,
  'XMLLIBSUBDIRS': 'xml xml/dom xml/etree xml/parsers xml/sax',
- 'abs_builddir': '/build/python3.10-mmQOQh/python3.10-3.10.6/build-shared',
- 'abs_srcdir': '/build/python3.10-mmQOQh/python3.10-3.10.6/build-shared/..',
+ 'abs_builddir': '/build/python3.10-cD1wDb/python3.10-3.10.7/build-shared',
+ 'abs_srcdir': '/build/python3.10-cD1wDb/python3.10-3.10.7/build-shared/..',
  'datarootdir': '/usr/share',
  'exec_prefix': '/usr',
  'prefix': '/usr',
