@@ -44,9 +44,13 @@ import subprocess
 import re
 import DistUpgrade.DistUpgradeCache
 from gettext import gettext as _
+try:
+    from launchpadlib.launchpad import Launchpad
+except ImportError:
+    Launchpad = None
 
 SYNAPTIC_PINFILE = "/var/lib/synaptic/preferences"
-CHANGELOGS_POOL = "http://changelogs.ubuntu.com/changelogs/pool/"
+CHANGELOGS_POOL = "https://changelogs.ubuntu.com/changelogs/pool/"
 CHANGELOGS_URI = CHANGELOGS_POOL + "%s/%s/%s/%s_%s/%s"
 
 
@@ -80,8 +84,9 @@ class MyCache(DistUpgrade.DistUpgradeCache.MyCache):
         # on broken packages, try to fix via saveDistUpgrade()
         if self._depcache.broken_count > 0:
             self.saveDistUpgrade()
-        assert (self._depcache.broken_count == 0 and
-                self._depcache.del_count == 0)
+        assert (self._depcache.broken_count == 0
+                and self._depcache.del_count == 0)
+        self.launchpad = None
         # generate versioned_kernel_pkgs_regexp for later use
         apt_versioned_kernel_pkgs = apt_pkg.config.value_list(
             "APT::VersionedKernelPackages")
@@ -137,6 +142,10 @@ class MyCache(DistUpgrade.DistUpgradeCache.MyCache):
     def keep_count(self):
         return self._depcache.keep_count
 
+    @property
+    def del_count(self):
+        return self._depcache.del_count
+
     def _check_dependencies(self, target, deps):
         """Return True if any of the dependencies in deps match target."""
         # TODO: handle virtual packages
@@ -145,8 +154,8 @@ class MyCache(DistUpgrade.DistUpgradeCache.MyCache):
                 continue
             match = True
             for base_dep in dep_or:
-                if (base_dep.name != target.package.shortname or
-                    not apt_pkg.check_dep(
+                if (base_dep.name != target.package.shortname
+                    or not apt_pkg.check_dep(
                         target.version, base_dep.relation, base_dep.version)):
                     match = False
             if match:
@@ -161,8 +170,8 @@ class MyCache(DistUpgrade.DistUpgradeCache.MyCache):
             candidate = cpkg.candidate
             if candidate is not None:
                 if (self._check_dependencies(
-                        target, candidate.get_dependencies("Conflicts")) and
-                    self._check_dependencies(
+                        target, candidate.get_dependencies("Conflicts"))
+                    and self._check_dependencies(
                         target, candidate.get_dependencies("Replaces"))):
                     logging.info(
                         "%s Conflicts/Replaces %s; allowing removal" % (
@@ -183,8 +192,8 @@ class MyCache(DistUpgrade.DistUpgradeCache.MyCache):
                     wouldDelete -= 1
         if wouldDelete > 0:
             self.clear()
-            assert (self._depcache.broken_count == 0 and
-                    self._depcache.del_count == 0)
+            assert (self._depcache.broken_count == 0
+                    and self._depcache.del_count == 0)
         else:
             assert self._depcache.broken_count == 0
         #self._apply_dselect_upgrade()
@@ -212,10 +221,9 @@ class MyCache(DistUpgrade.DistUpgradeCache.MyCache):
         # use the section of the candidate as a starting point
         section = pkg._pcache._depcache.get_candidate_ver(pkg._pkg).section
 
-        # get the source version, start with the binaries version
-        srcver_epoch = pkg.candidate.version
+        # get the source version
+        srcver_epoch = pkg.candidate.source_version
         srcver = self._strip_epoch(srcver_epoch)
-        #print("bin: %s" % binver)
 
         split_section = section.split("/")
         if len(split_section) > 1:
@@ -240,7 +248,7 @@ class MyCache(DistUpgrade.DistUpgradeCache.MyCache):
         # and so its possible to do a man-in-the-middle attack to steal the
         # credentials
         res = urlsplit(uri)
-        if res.scheme == "https" and res.username != "":
+        if res.scheme == "https" and res.username:
             raise HttpsChangelogsUnsupportedError(
                 "https locations with username/password are not"
                 "supported to fetch changelogs")
@@ -250,7 +258,7 @@ class MyCache(DistUpgrade.DistUpgradeCache.MyCache):
         #print(changelog.read())
         # do only get the lines that are new
         alllines = ""
-        regexp = "^%s \((.*)\)(.*)$" % (re.escape(srcpkg))
+        regexp = "^%s \\((.*)\\)(.*)$" % (re.escape(srcpkg))
 
         while True:
             line = changelog.readline().decode("UTF-8", "replace")
@@ -274,18 +282,64 @@ class MyCache(DistUpgrade.DistUpgradeCache.MyCache):
                 #
                 # for NEWS.Debian we do require the changelogver > installed
                 if strict_versioning:
-                    if (installed and
-                            apt_pkg.version_compare(changelogver,
-                                                    installed) < 0):
+                    if (installed
+                            and apt_pkg.version_compare(changelogver,
+                                                        installed) < 0):
                         break
                 else:
-                    if (installed and
-                            apt_pkg.version_compare(changelogver,
-                                                    installed) == 0):
+                    if (installed
+                            and apt_pkg.version_compare(changelogver,
+                                                        installed) == 0):
                         break
             alllines = alllines + line
         return alllines
 
+    def _extract_ppa_changelog_uri(self, name):
+        """Return the changelog URI from the Launchpad API
+
+        Return None in case of an error.
+        """
+        if not Launchpad:
+            logging.warning("Launchpadlib not available, cannot retrieve PPA "
+                            "changelog")
+            return None
+
+        cdt = self[name].candidate
+        for uri in cdt.uris:
+            if urlsplit(uri).hostname != 'ppa.launchpad.net':
+                continue
+            match = re.search('http.*/(.*)/(.*)/ubuntu/.*', uri)
+            if match is not None:
+                user, ppa = match.group(1), match.group(2)
+                break
+        else:
+            logging.error("Unable to find a valid PPA candidate URL.")
+            return
+
+        # Login on launchpad if we are not already
+        if self.launchpad is None:
+            self.launchpad = Launchpad.login_anonymously('update-manager',
+                                                         'production',
+                                                         version='devel')
+
+        archive = self.launchpad.archives.getByReference(
+            reference='~%s/ubuntu/%s' % (user, ppa)
+        )
+        if archive is None:
+            logging.error("Unable to retrieve the archive from the Launchpad "
+                          "API.")
+            return
+
+        spphs = archive.getPublishedSources(source_name=cdt.source_name,
+                                            exact_match=True,
+                                            version=cdt.source_version)
+        if not spphs:
+            logging.error("No published sources were retrieved from the "
+                          "Launchpad API.")
+            return
+
+        return spphs[0].changelogUrl()
+
     def _guess_third_party_changelogs_uri_by_source(self, name):
         pkg = self[name]
         deb_uri = pkg.candidate.uri
@@ -299,7 +353,7 @@ class MyCache(DistUpgrade.DistUpgradeCache.MyCache):
         if "(" in srcrec:
             srcver = srcrec.split("(")[1].rstrip(")")
         else:
-            srcver = pkg.candidate.version
+            srcver = pkg.candidate.source_version
         base_uri = deb_uri.rpartition("/")[0]
         return base_uri + "/%s_%s.changelog" % (srcpkg, srcver)
 
@@ -320,7 +374,7 @@ class MyCache(DistUpgrade.DistUpgradeCache.MyCache):
         self.get_changelog(name)
         try:
             lock.release()
-        except Exception as e:
+        except Exception:
             pass
 
     def get_news(self, name):
@@ -332,14 +386,26 @@ class MyCache(DistUpgrade.DistUpgradeCache.MyCache):
         if news:
             self.all_news[name] = news
 
-    def _fetch_changelog_for_third_party_package(self, name):
+    def _fetch_changelog_for_third_party_package(self, name, origins):
+        # Special case for PPAs
+        changelogs_uri_ppa = None
+        for origin in origins:
+            if origin.origin.startswith('LP-PPA-'):
+                try:
+                    changelogs_uri_ppa = self._extract_ppa_changelog_uri(name)
+                    break
+                except Exception:
+                    logging.exception("Unable to connect to the Launchpad "
+                                      "API.")
         # Try non official changelog location
         changelogs_uri_binary = \
             self._guess_third_party_changelogs_uri_by_binary(name)
         changelogs_uri_source = \
             self._guess_third_party_changelogs_uri_by_source(name)
         error_message = ""
-        for changelogs_uri in [changelogs_uri_binary, changelogs_uri_source]:
+        for changelogs_uri in [changelogs_uri_ppa,
+                               changelogs_uri_binary,
+                               changelogs_uri_source]:
             if changelogs_uri:
                 try:
                     changelog = self._get_changelog_or_news(
@@ -367,11 +433,11 @@ class MyCache(DistUpgrade.DistUpgradeCache.MyCache):
             (name, getattr(self[name].installed, "version", None),
              self[name].candidate.version)
         if self.CHANGELOG_ORIGIN not in [o.origin for o in origins]:
-            self._fetch_changelog_for_third_party_package(name)
+            self._fetch_changelog_for_third_party_package(name, origins)
             return
         # fixup epoch handling version
         srcpkg = self[name].candidate.source_name
-        srcver_epoch = self[name].candidate.version.replace(':', '%3A')
+        srcver_epoch = self[name].candidate.source_version.replace(':', '%3A')
         try:
             changelog = self._get_changelog_or_news(name, "changelog")
             if len(changelog) == 0:
@@ -381,7 +447,7 @@ class MyCache(DistUpgrade.DistUpgradeCache.MyCache):
                               "%s/%s/+changelog\n"
                               "until the changes become available or try "
                               "again later.") % (srcpkg, srcver_epoch)
-        except HTTPError as e:
+        except HTTPError:
             changelog = _("The list of changes is not available yet.\n\n"
                           "Please use http://launchpad.net/ubuntu/+source/"
                           "%s/%s/+changelog\n"
