@@ -1,29 +1,66 @@
 """ansible enables running on first boot either ansible-pull"""
 import abc
-import logging
 import os
 import re
 import sys
-import sysconfig
 from copy import deepcopy
+from logging import getLogger
+from textwrap import dedent
 from typing import Optional
 
-from cloudinit import subp
 from cloudinit.cloud import Cloud
 from cloudinit.config import Config
-from cloudinit.config.schema import MetaSchema
+from cloudinit.config.schema import MetaSchema, get_meta_doc
 from cloudinit.distros import ALL_DISTROS, Distro
 from cloudinit.settings import PER_INSTANCE
+from cloudinit.subp import subp, which
 from cloudinit.util import Version, get_cfg_by_path
 
 meta: MetaSchema = {
     "id": "cc_ansible",
+    "name": "Ansible",
+    "title": "Configure ansible for instance",
     "frequency": PER_INSTANCE,
     "distros": [ALL_DISTROS],
     "activate_by_schema_keys": ["ansible"],
-}  # type: ignore
+    "description": dedent(
+        """\
+        This module provides ``ansible`` integration for
+        augmenting cloud-init's configuration of the local
+        node.
 
-LOG = logging.getLogger(__name__)
+
+        This module installs ansible during boot and
+        then uses ``ansible-pull`` to run the playbook
+        repository at the remote URL.
+        """
+    ),
+    "examples": [
+        dedent(
+            """\
+            ansible:
+              package_name: ansible-core
+              install_method: distro
+              pull:
+                url: "https://github.com/holmanb/vmboot.git"
+                playbook_name: ubuntu.yml
+            """
+        ),
+        dedent(
+            """\
+            ansible:
+              package_name: ansible-core
+              install_method: pip
+              pull:
+                url: "https://github.com/holmanb/vmboot.git"
+                playbook_name: ubuntu.yml
+            """
+        ),
+    ],
+}
+
+__doc__ = get_meta_doc(meta)
+LOG = getLogger(__name__)
 CFG_OVERRIDE = "ansible_config"
 
 
@@ -32,12 +69,12 @@ class AnsiblePull(abc.ABC):
         self.cmd_pull = ["ansible-pull"]
         self.cmd_version = ["ansible-pull", "--version"]
         self.distro = distro
-        self.env = {}
+        self.env = os.environ
         self.run_user: Optional[str] = None
 
         # some ansible modules directly reference os.environ["HOME"]
         # and cloud-init might not have that set, default: /root
-        self.env["HOME"] = os.environ.get("HOME", "/root")
+        self.env["HOME"] = self.env.get("HOME", "/root")
 
     def get_version(self) -> Optional[Version]:
         stdout, _ = self.do_as(self.cmd_version)
@@ -62,7 +99,7 @@ class AnsiblePull(abc.ABC):
         return self.distro.do_as(command, self.run_user, **kwargs)
 
     def subp(self, command, **kwargs):
-        return subp.subp(command, update_env=self.env, **kwargs)
+        return subp(command, env=self.env, **kwargs)
 
     @abc.abstractmethod
     def is_installed(self):
@@ -98,19 +135,8 @@ class AnsiblePullPip(AnsiblePull):
             try:
                 import pip  # noqa: F401
             except ImportError:
-                self.distro.install_packages([self.distro.pip_package_name])
-            cmd = [
-                sys.executable,
-                "-m",
-                "pip",
-                "install",
-            ]
-            if os.path.exists(
-                os.path.join(
-                    sysconfig.get_path("stdlib"), "EXTERNALLY-MANAGED"
-                )
-            ):
-                cmd.append("--break-system-packages")
+                self.distro.install_packages(self.distro.pip_package_name)
+            cmd = [sys.executable, "-m", "pip", "install"]
             if self.run_user:
                 cmd.append("--user")
             self.do_as([*cmd, "--upgrade", "pip"])
@@ -124,10 +150,10 @@ class AnsiblePullPip(AnsiblePull):
 class AnsiblePullDistro(AnsiblePull):
     def install(self, pkg_name: str):
         if not self.is_installed():
-            self.distro.install_packages([pkg_name])
+            self.distro.install_packages(pkg_name)
 
     def is_installed(self) -> bool:
-        return bool(subp.which("ansible"))
+        return bool(which("ansible"))
 
 
 def handle(name: str, cfg: Config, cloud: Cloud, args: list) -> None:
