@@ -11,7 +11,6 @@
 import abc
 import copy
 import json
-import logging
 import os
 import pickle
 import re
@@ -19,7 +18,9 @@ from collections import namedtuple
 from enum import Enum, unique
 from typing import Any, Dict, List, Optional, Tuple
 
-from cloudinit import atomic_helper, dmi, importer, net, type_utils
+from cloudinit import dmi, importer
+from cloudinit import log as logging
+from cloudinit import net, type_utils
 from cloudinit import user_data as ud
 from cloudinit import util
 from cloudinit.atomic_helper import write_json
@@ -261,9 +262,7 @@ class DataSource(CloudInitPickleMixin, metaclass=abc.ABCMeta):
     # N-tuple of keypaths or keynames redact from instance-data.json for
     # non-root users
     sensitive_metadata_keys: Tuple[str, ...] = (
-        "combined_cloud_config",
         "merged_cfg",
-        "merged_system_cfg",
         "security-credentials",
         "userdata",
         "user-data",
@@ -287,7 +286,7 @@ class DataSource(CloudInitPickleMixin, metaclass=abc.ABCMeta):
         self.paths = paths
         self.userdata = None
         self.metadata: dict = {}
-        self.userdata_raw: Optional[str] = None
+        self.userdata_raw = None
         self.vendordata = None
         self.vendordata2 = None
         self.vendordata_raw = None
@@ -351,7 +350,10 @@ class DataSource(CloudInitPickleMixin, metaclass=abc.ABCMeta):
                 self,
             )
             return True
-        elif self.sys_cfg.get("datasource_list", []) == [self.dsname]:
+        elif self.sys_cfg.get("datasource_list", []) in (
+            [self.dsname],
+            [self.dsname, "None"],
+        ):
             LOG.debug(
                 "Machine is configured to run on single datasource %s.", self
             )
@@ -476,14 +478,7 @@ class DataSource(CloudInitPickleMixin, metaclass=abc.ABCMeta):
         instance_data["ds"]["_doc"] = EXPERIMENTAL_TEXT
         # Add merged cloud.cfg and sys info for jinja templates and cli query
         instance_data["merged_cfg"] = copy.deepcopy(self.sys_cfg)
-        instance_data["merged_cfg"][
-            "_doc"
-        ] = "DEPRECATED: Use merged_system_cfg. Will be dropped from 24.1"
-        # Deprecate merged_cfg to a more specific key name merged_system_cfg
-        instance_data["merged_system_cfg"] = copy.deepcopy(
-            instance_data["merged_cfg"]
-        )
-        instance_data["merged_system_cfg"]["_doc"] = (
+        instance_data["merged_cfg"]["_doc"] = (
             "Merged cloud-init system config from /etc/cloud/cloud.cfg and"
             " /etc/cloud/cloud.cfg.d/"
         )
@@ -491,7 +486,7 @@ class DataSource(CloudInitPickleMixin, metaclass=abc.ABCMeta):
         instance_data.update(self._get_standardized_metadata(instance_data))
         try:
             # Process content base64encoding unserializable values
-            content = atomic_helper.json_dumps(instance_data)
+            content = util.json_dumps(instance_data)
             # Strip base64: prefix and set base64_encoded_keys list.
             processed_data = process_instance_metadata(
                 json.loads(content),
