@@ -5,12 +5,10 @@ from typing import Dict, List, Type  # noqa: F401
 from uaclient import messages
 from uaclient.config import UAConfig
 from uaclient.entitlements import fips
-from uaclient.entitlements.anbox import AnboxEntitlement
 from uaclient.entitlements.base import UAEntitlement  # noqa: F401
 from uaclient.entitlements.cc import CommonCriteriaEntitlement
 from uaclient.entitlements.cis import CISEntitlement
 from uaclient.entitlements.esm import ESMAppsEntitlement, ESMInfraEntitlement
-from uaclient.entitlements.landscape import LandscapeEntitlement
 from uaclient.entitlements.livepatch import LivepatchEntitlement
 from uaclient.entitlements.realtime import RealtimeKernelEntitlement
 from uaclient.entitlements.ros import ROSEntitlement, ROSUpdatesEntitlement
@@ -18,14 +16,12 @@ from uaclient.exceptions import EntitlementNotFoundError
 from uaclient.util import is_config_value_true
 
 ENTITLEMENT_CLASSES = [
-    AnboxEntitlement,
     CommonCriteriaEntitlement,
     CISEntitlement,
     ESMAppsEntitlement,
     ESMInfraEntitlement,
     fips.FIPSEntitlement,
     fips.FIPSUpdatesEntitlement,
-    LandscapeEntitlement,
     LivepatchEntitlement,
     RealtimeKernelEntitlement,
     ROSEntitlement,
