@@ -27,10 +27,9 @@ import logging
 import sys
 import time
 
-from uaclient import contract, defaults, messages, system, util
+from uaclient import contract, defaults, system
 from uaclient.api.u.pro.status.is_attached.v1 import _is_attached
 from uaclient.config import UAConfig
-from uaclient.files.machine_token import get_machine_token_file
 
 # We consider the past release for LTSs to be the last LTS,
 # because we don't have any services available on non-LTS.
@@ -42,59 +41,52 @@ current_codename_to_past_codename = {
     "bionic": "xenial",
     "focal": "bionic",
     "jammy": "focal",
-    "mantic": "lunar",
-    "noble": "jammy",
+    "kinetic": "jammy",
 }
 
-LOG = logging.getLogger(util.replace_top_level_logger_name(__name__))
-
 
 def process_contract_delta_after_apt_lock(cfg: UAConfig) -> None:
-    LOG.debug("Check whether to upgrade-lts-contract")
+    logging.debug("Check whether to upgrade-lts-contract")
     if not _is_attached(cfg).is_attached:
-        LOG.debug("Skipping upgrade-lts-contract. Machine is unattached")
+        logging.debug("Skipping upgrade-lts-contract. Machine is unattached")
         return
-    LOG.debug("Starting upgrade-lts-contract.")
     out, _err = system.subp(["lsof", "/var/lib/apt/lists/lock"], rcs=[0, 1])
+    msg = "Starting upgrade-lts-contract."
     if out:
-        print(messages.RELEASE_UPGRADE_APT_LOCK_HELD_WILL_WAIT)
+        msg += " Retrying every 10 seconds waiting on released apt lock"
+    print(msg)
+    logging.debug(msg)
 
     current_release = system.get_release_info().series
-    machine_token_file = get_machine_token_file(cfg)
 
     past_release = current_codename_to_past_codename.get(current_release)
     if past_release is None:
-        print(
-            messages.RELEASE_UPGRADE_NO_PAST_RELEASE.format(
-                release=current_release
-            )
-        )
-        LOG.warning(
-            "Could not find past release for %s. Current known releases: %r.",
-            current_release,
-            current_codename_to_past_codename,
-        )
+        msg = "Could not find past release for: {}".format(current_release)
+        print(msg)
+        logging.warning(msg)
         sys.exit(1)
 
-    past_entitlements = machine_token_file.entitlements(series=past_release)
-    new_entitlements = machine_token_file.entitlements()
+    past_entitlements = UAConfig(
+        series=past_release,
+    ).machine_token_file.entitlements
+    new_entitlements = UAConfig(
+        series=current_release,
+    ).machine_token_file.entitlements
 
     retry_count = 0
     while out:
         # Loop until apt hold is released at the end of `do-release-upgrade`
-        LOG.debug("Detected that apt lock is held. Sleeping 10 seconds.")
         time.sleep(10)
         out, _err = system.subp(
             ["lsof", "/var/lib/apt/lists/lock"], rcs=[0, 1]
         )
         retry_count += 1
 
-    LOG.debug(
-        "upgrade-lts-contract processing contract deltas: %s -> %s",
-        past_release,
-        current_release,
+    msg = "upgrade-lts-contract processing contract deltas: {} -> {}".format(
+        past_release, current_release
     )
-    print(messages.RELEASE_UPGRADE_STARTING)
+    print(msg)
+    logging.debug(msg)
 
     contract.process_entitlements_delta(
         cfg=cfg,
@@ -103,8 +95,9 @@ def process_contract_delta_after_apt_lock(cfg: UAConfig) -> None:
         allow_enable=True,
         series_overrides=False,
     )
-    LOG.debug("upgrade-lts-contract succeeded after %s retries", retry_count)
-    print(messages.RELEASE_UPGRADE_SUCCESS)
+    msg = "upgrade-lts-contract succeeded after {} retries".format(retry_count)
+    print(msg)
+    logging.debug(msg)
 
 
 def remove_private_esm_apt_cache():
