@@ -1,32 +1,39 @@
-"""
-Send reports about ubuntu-desktop-installer to the correct Launchpad project.
-
-"""
+"""Send reports about ubuntu-desktop-installer to the correct Launchpad
+project."""
 import os
 
 from apport import hookutils
 
 
-def add_info(report, ui):
+def add_info(report, unused_ui):
 
-    udilog = os.path.realpath("/var/log/installer/ubuntu_desktop_installer.log")
+    udilog = os.path.realpath(
+        "/var/log/installer/ubuntu_desktop_installer.log"
+    )
     hookutils.attach_file_if_exists(report, udilog, "UdiLog")
 
-    report['SourcePackage'] = 'ubuntu-desktop-installer'
+    report["SourcePackage"] = "ubuntu-desktop-installer"
     # rewrite this section so the report goes to the project in Launchpad
     report[
         "CrashDB"
-    ] = """{
-       "impl": "launchpad",
-       "project": "ubuntu-desktop-installer",
-       "bug_pattern_url": "http://people.canonical.com/~ubuntu-archive/bugpatterns/bugpatterns.xml"
-    }"""
+    ] = """\
+{
+    "impl": "launchpad",
+    "project": "ubuntu-desktop-installer",
+    "bug_pattern_url": "http://people.canonical.com/"
+    "~ubuntu-archive/bugpatterns/bugpatterns.xml",
+}
+"""
 
-    subiquitylog = os.path.realpath("/var/log/installer/subiquity-server-debug.log")
+    subiquitylog = os.path.realpath(
+        "/var/log/installer/subiquity-server-debug.log"
+    )
     hookutils.attach_file_if_exists(report, subiquitylog, "SubiquityLog")
 
     hookutils.attach_file_if_exists(
-        report, "/var/log/installer/subiquity-curtin-install.conf", "CurtinConfig"
+        report,
+        "/var/log/installer/subiquity-curtin-install.conf",
+        "CurtinConfig",
     )
     hookutils.attach_file_if_exists(
         report, "/var/log/curtin/install.log", "CurtinLog"
