@@ -99,6 +99,10 @@ class Parser:
         lib.netplan_parser_load_nullable_fields.argtypes = [_NetplanParserP, c_int, _GErrorPP]
         lib.netplan_parser_load_nullable_fields.restype = c_int
 
+        lib.netplan_parser_load_nullable_overrides.argtypes =\
+            [_NetplanParserP, c_int, c_char_p, _GErrorPP]
+        lib.netplan_parser_load_nullable_overrides.restype = c_int
+
         cls._abi_loaded = True
 
     def __init__(self):
@@ -120,6 +124,10 @@ class Parser:
     def load_nullable_fields(self, input_file: IO):
         _checked_lib_call(lib.netplan_parser_load_nullable_fields, self._ptr, input_file.fileno())
 
+    def load_nullable_overrides(self, input_file: IO, constraint: str):
+        _checked_lib_call(lib.netplan_parser_load_nullable_overrides,
+                          self._ptr, input_file.fileno(), constraint.encode('utf-8'))
+
 
 class State:
     _abi_loaded = False
